/* USER CODE BEGIN Header */
/**
 ******************************************************************************
 * @file           : main.c
 * @brief          : Main program body
 ******************************************************************************
 * @attention
 *
 * Copyright (c) 2022 STMicroelectronics.
 * All rights reserved.
 *
 * This software is licensed under terms that can be found in the LICENSE file
 * in the root directory of this software component.
 * If no LICENSE file comes with this software, it is provided AS-IS.
 *
 ******************************************************************************
 */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "i2c.h"
#include "tim.h"
#include "usart.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <stdio.h>
#include <string.h>
#include "mlx90614.h"

#include "ssd1306.h"
#include "ssd1306_tests.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */
typedef struct
{
	char name[20];
	float emissivity;
} BodyEmissivity_t;
/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define TIME_TEST
#define TEXT_LINE_LENGTH 30
#define NUMBER_OF_MATERIAL_TYPES 5
#define OLED_REFRESH_PERIOD 250
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
char text_line[TEXT_LINE_LENGTH];
uint8_t text_len;
uint16_t enc_count;

float ObjectTemperature;
float AmbientTemperature;
float Emissivity;

BodyEmissivity_t MaterialTypes[NUMBER_OF_MATERIAL_TYPES] =
{ [0].name = "Skin", [0].emissivity = 0.985, [1].name = "Water",
		[1].emissivity = 0.96, [2].name = "Paper", [2].emissivity = 0.94,
		[3].name = "Brick", [3].emissivity = 0.45, [4].name = "Paint",
		[4].emissivity = 0.92 };

uint16_t current_material = 0;
uint8_t toggle_visibility = 0;

MLX90614_STATUS sensor_status;

uint32_t SoftTimerOLED;
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_USART2_UART_Init();
  MX_I2C1_Init();
  MX_TIM2_Init();
  /* USER CODE BEGIN 2 */
	HAL_GPIO_WritePin(TEST_GPIO_Port, TEST_Pin, 0);
	MLX90614_Init(&hi2c1);
	MLX90614_SetEmissivity(MaterialTypes[current_material].emissivity); // Human skin


	// HMI encoder
	__HAL_TIM_SET_AUTORELOAD(&htim2, (NUMBER_OF_MATERIAL_TYPES - 1));
	HAL_TIM_Encoder_Start(&htim2, TIM_CHANNEL_ALL);

	// OLED welcome message
	ssd1306_Init();
	ssd1306_Fill(Black);
	ssd1306_SetCursor(0, 5);
	ssd1306_WriteString("MLX90614 pyrometer", Font_6x8, White);
	ssd1306_SetCursor(0, 15);
	ssd1306_WriteString("Lib. by msalamon.pl", Font_6x8, White);
	ssd1306_UpdateScreen();

	SoftTimerOLED = HAL_GetTick();
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
	while (1)
	{
		if ((HAL_GetTick() - SoftTimerOLED) > OLED_REFRESH_PERIOD)
		{
			SoftTimerOLED = HAL_GetTick();
			MLX90614_ReadAmbientTemperature(&AmbientTemperature);
#ifdef TIME_TEST
			HAL_GPIO_WritePin(TEST_GPIO_Port, TEST_Pin, 1);
#endif
			MLX90614_ReadObjectTemperature(&ObjectTemperature);
#ifdef TIME_TEST
			HAL_GPIO_WritePin(TEST_GPIO_Port, TEST_Pin, 0);
#endif
			MLX90614_GetEmissivity(&Emissivity);

			memset(text_line, 0x00, TEXT_LINE_LENGTH);
			enc_count = __HAL_TIM_GET_COUNTER(&htim2);
			text_len = sprintf(text_line, "Emissivity: %.3f  \n\r", Emissivity);
			HAL_UART_Transmit(&huart2, (uint8_t*) text_line, text_len, 20);
			ssd1306_SetCursor(0, 25);
			ssd1306_WriteString(text_line, Font_6x8, White);

			memset(text_line, 0x00, TEXT_LINE_LENGTH);
			text_len = sprintf(text_line, "Ambient: %.1f deg. C \n\r",
					AmbientTemperature);
			HAL_UART_Transmit(&huart2, (uint8_t*) text_line, text_len, 20);
			ssd1306_SetCursor(0, 35);
			ssd1306_WriteString(text_line, Font_6x8, White);

			memset(text_line, 0x00, TEXT_LINE_LENGTH);
			text_len = sprintf(text_line, "Object:  %.1f deg. C \n\n\r",
					ObjectTemperature);
			HAL_UART_Transmit(&huart2, (uint8_t*) text_line, text_len, 20);
			ssd1306_SetCursor(0, 45);
			ssd1306_WriteString(text_line, Font_6x8, White);

			if (enc_count == current_material)
			{
				ssd1306_SetCursor(0, 55);
				ssd1306_WriteString("                ", Font_6x8, White); // :)
				ssd1306_SetCursor(0, 55);
				ssd1306_WriteString("         ", Font_6x8, White);
				ssd1306_WriteString(MaterialTypes[enc_count].name, Font_6x8,
						White);
			}
			else
			{
				if (0 == toggle_visibility)
				{
					toggle_visibility = 1;
					ssd1306_SetCursor(0, 55);
					ssd1306_WriteString("                ", Font_6x8, White); // :)
					ssd1306_SetCursor(0, 55);
					ssd1306_WriteString("         ", Font_6x8, White);
					ssd1306_WriteString(MaterialTypes[enc_count].name, Font_6x8,
							White);
				}
				else
				{
					toggle_visibility = 0;
					ssd1306_SetCursor(0, 55);
					ssd1306_WriteString("                ", Font_6x8, White); // :)
				}
			}
			ssd1306_UpdateScreen();
		}
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
	}
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  if (HAL_PWREx_ControlVoltageScaling(PWR_REGULATOR_VOLTAGE_SCALE1) != HAL_OK)
  {
    Error_Handler();
  }

  /** Configure LSE Drive Capability
  */
  HAL_PWR_EnableBkUpAccess();
  __HAL_RCC_LSEDRIVE_CONFIG(RCC_LSEDRIVE_LOW);

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_LSE|RCC_OSCILLATORTYPE_MSI;
  RCC_OscInitStruct.LSEState = RCC_LSE_ON;
  RCC_OscInitStruct.MSIState = RCC_MSI_ON;
  RCC_OscInitStruct.MSICalibrationValue = 0;
  RCC_OscInitStruct.MSIClockRange = RCC_MSIRANGE_6;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_MSI;
  RCC_OscInitStruct.PLL.PLLM = 1;
  RCC_OscInitStruct.PLL.PLLN = 16;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV7;
  RCC_OscInitStruct.PLL.PLLQ = RCC_PLLQ_DIV2;
  RCC_OscInitStruct.PLL.PLLR = RCC_PLLR_DIV2;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK)
  {
    Error_Handler();
  }

  /** Enable MSI Auto calibration
  */
  HAL_RCCEx_EnableMSIPLLMode();
}

/* USER CODE BEGIN 4 */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
	if (GPIO_Pin == ENC_BUTTON_Pin)
	{
		sensor_status = MLX90614_SetEmissivity(
				MaterialTypes[enc_count].emissivity);
		if (MLX90614_OK == sensor_status)
		{
			current_material = enc_count;
		}
	}
}
/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
	/* User can add his own implementation to report the HAL error return state */
	__disable_irq();
	while (1)
	{
	}
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
